// Simon King (sjking@andrew.cmu.edu / simon@currentform.com)
// http://www.currentform.com/squircles/


// load pre-sorted image information files
String[] imageInfo = loadStrings("sorted_images.txt");
String[] serverInfo = loadStrings("sorted_server.txt");
String[] avgColorInfo = loadStrings("sorted_avgcolor.txt");
Circle[][] images = new Circle[256][256];
int mouseOverIndex;
int locX;
int locY;
PImage colorMap;

void setup() {
  size(757,256);
  background(255,241,241);
  cursor(HAND);
  drawInitialImageMap();
  colorMap = get(0, 0, 257, 256);
}

void draw() {

}

void refreshRect() {
  // this is way quicker than manually setting each pixel each time
  image(colorMap,0,0);
  stroke(255,255,0);
  noFill();
  // make sure the rect only highlights full 2x2 pixel squares
  int rectX = (locX % 2 == 0) ? locX-1 : locX;
  int rectY = (locY % 2 != 0) ? locY-2 : locY-1;
  rect(rectX-2, rectY, 11, 3);
}

void mousePressed() {
  noCursor();
}

void mouseReleased() {
  cursor(HAND); 
  getMouseIndex();
  displayHoverImage();
  refreshRect(); 
}

void mouseDragged() {
  noCursor();
  getMouseIndex();
  refreshRect();
}

void keyPressed() {
  if(key == CODED) { 
    if(keyCode == LEFT) {
      locX = locX - 2;
      ;
      locX = constrain(locX, 1, 247);
      refreshRect();
      displayHoverImage();
    }
    if(keyCode == RIGHT) {
      locX = locX + 2;
      locX = constrain(locX, 1, 247);
      refreshRect();
      displayHoverImage();
    }
    if(keyCode == UP) {
      locY = locY - 2;
      locY = constrain(locY, 1, 255);
      refreshRect();
      displayHoverImage();
    }
    if(keyCode == DOWN) {
      locY = locY + 2;
      locY = constrain(locY, 1, 255);
      refreshRect();
      displayHoverImage();
    }    
  } 
}

void getMouseIndex() {
  locX = constrain(mouseX, 1, 247);
  locY = constrain(mouseY, 1, 255);
}

void displayHoverImage() {
  if (mouseOverIndex >= 0) {
    int totalImagesToShow = 5;    
    for (int i=0; i<totalImagesToShow; i++) {
      int newLocX = constrain(locX + (i*2),0,255);
      mouseOverIndex = images[newLocX][locY].num;
      String url = getURL(mouseOverIndex,1);
      PImage currentImage = loadImage(url);
      image(currentImage,257 + (100*i),78, 100, 100);
    }
  } 
}

String getURL(int index, int imageSize) {
  String url = "";
  String code = imageInfo[index];
  String server = serverInfo[index];
  if (imageSize == 1) {
    url = code + "_t.jpg";
  }
  return url;
}

void drawInitialImageMap() {
  int col = 0;
  int row = 0;
  int direction = 0;
  for (int i=0; i<imageInfo.length; i++) {
    // this is made more complicated because we want to loop the rows back
    // and forth, not starting over at the beginning of each row. This is
    // so we can keep like images near each other
    if (i > 0) {
      if (i%128 == 0) {
        row = row + 2;
        direction = (direction == 0) ? 1 : 0;
        col = (direction == 0) ? 0 : 255; // doubled because we're doing 4x4 not single pixel
      } 
      else {
        col = (direction == 0) ? col + 2 : col - 2;
      }
    }
    color c = int(avgColorInfo[i]);
    // set the pixel
    set(col,row,c);
    // figure out how to draw a 4x4 of this color based on the direction we're drawing
    int colmodifier = (direction == 0) ? 1 : -1;
    set(col+colmodifier,row,c);
    set(col,row+1,c);
    set(col+colmodifier,row+1,c);

    Circle circ = new Circle(i);
    images[col][row] = circ;
    images[col+colmodifier][row] = circ;
    images[col][row+1] = circ;
    images[col+colmodifier][row+1] = circ;
  }
}

class Circle {
  // this wouldn't need to be a class but this way it's 
  //extensible in the future if I want to do more with it
  int num;

  Circle(int num) {
    this.num = num;
  }
}
